/*  Copyright (c)   Mar, 1986   UMMC.  All rights reserved

Filename:       clip.h

Abstract:       This file contains all structure and parameter definitions relevant to the CLIP
            programming environment except the error code definitions.

Environment:    UM/CLIP

Revision History:
    Rev #      Date    Auth     Reason
    -----   ---------  -----    ----------------------------------
    0.0     13-mar-86   js      original implementation
    0.1     19-mar-86   daf     remove boolean, true, false definitions
    0.2     24-mar-86   js      defined Pi
=========================================================================*/

/* define system wide values */

#define     Pi      3.1415926535

/* define C-arm structure */

struct c_arm
    {
    INT     radius;     /* radius of C-arm in cm */
    INT     alpha;      /* current angle alpha of C-arm angulation in degrees*/
    INT     maxalpha;       /* upper limit on alpha */
    INT     minalpha;       /* lower limit on alpha */
    INT     beta;           /* current angle beta of C-arm angulations in degrees */
    INT     maxbeta;        /* upper limit on beta */
    INT     minbeta;        /* lower limit on beta */
    INT     sd;         /* source-detector distance in cm */
    INT     maxsd;      /* maximum sd */
    INT     minsd;      /* minimum sd */
    INT     so;         /* source-object (focal spot) distance in cm */
    INT     od;         /* object-detector distance in cm */
    INT     maxod;      /* maximum od */
    INT     minod;      /* minimum od */
    FLOAT   iirad;          /* radius of image intensifier in cm */
    FLOAT   psi;            /* solid angle of X-ray cone beam in steradians */
    FLOAT   magfact;        /* magnification factor at iso-center */
    };
typedef struct c_arm c_arm;


/* define imaging chain structure */

struct ichain
    {
    /* just a stub for now */
    INT     stub;
    };
typedef struct ichain ichain;


/* define table structure */

struct table
    {
    INT     x;          /* x position of reference point on table relative to iso-center in cm */
    INT     y;          /* y position of reference point */
    INT     z;          /* z position of reference point */
    INT     zdisp;      /* vertical displacement of cylindrical model of table/patient in cm */
    INT     radius;     /* radius of cylindrical model in cm */
    INT     length;     /* length of cylindrical model in cm */
    };
typedef struct table table;



/* define gantry structure */

struct gantry
    {
    c_arm   *ap;            /* pointer to AP C-arm structure */
    c_arm   *lat;       /* pointer to LAT C-arm structure */
    ichain  *apich;     /* pointer to AP imaging chain structure */
    ichain  *latich;        /* pointer to LAT imaging chain structure */
    int     park;       /* parked/in use switch for LAT C-arm */
    int     locked;     /* locked/independent switch for C-arms */
    table       *tab;       /* pointer to table structure */
    };
typedef struct gantry gantry;
