;
; Chatter by Donald Brown
;  (c) 1985 CE Software
;  This program allows several people on an AppleTalk network
;  to send messages to each other
;
;  You may use the Chatter program as you see fit, so long as CE Software's
;  copyright notices remain intact.  We do request $25.00 if you use the
;  Chatter source code to help you learn about AppleTalk and/or develop
;  your own Appletalk software.  Also, if you use major parts of our
;  Appletalk routines in your programs, we request acknowledgement.
;
;  Chatter is a product of:
;	CE Software
;	801 73rd Street
;	Des Moines, IA  50312
;	  (515) 224-1995.
;
;  This program is based on the "Window" sample program on the
;  Macintosh Development System.
;
;  Most of the Appletalk programming appears at the end.  I have attempted
;  to flag the changes I made inside the "Window" code by a commented line
;  of stars:
;*********************************************************************
;
;  "Window" kept most of its information in registers.  Since I like to
;  use registers in my routines, I have moved most of that information into
;  global storage.  Most of the changes of replacing a register reference
;  with a global variable are not flagged with the stars.
;
; DESCRIPTION OF CHATTER:
;   Chatter is a program designed to work with the AppleTalk network.
;   It was written as a learning program, and a shell to build future
;   AppleTalk programs from.
;
;   Chatter's screen is broken into three parts:  A list of all names
;   (other than the user himself) that are on the network running the
;   chatter program, a textedit record where the user types in messages
;   to transmit, and a textedit record that emulates our "screen" which
;   shows all the incoming messages.
;
;   All AppleTalk calls are called asynchronously.  With a few exceptions
;   (getting and releasing the socket, registering and removing the user
;   name), Chatter has these calls execute a completion routine that will
;   post an application-defined event when they are done.  This allows
;   Chatter to handle the finished events as part of its main event loop.
;
;   Chatter sends three types of messages to other computers on the node:
;   "hello" messages (I have just entered the network, put me in your list),
;   "goodbye" messages (I am quitting, remove me from your list), and
;   regular messages (print this on your screen).  These messages are sent
;   as requests from the computer sending the message to the computer
;   receiving the message.  The receiving computer must send a response
;   back to the sending computer, even though (in our case) the response
;   means only "Yes, I got it."
;
; KNOWN FLAWS:  There are some known potential problems that Chatter does
;  not handle well.  This list is not inclusive, I'm sure there are flaws
;  I haven't thought of.  Please feel free to write me with other flaws
;  you discover.
;
;  1)  It assumes it's running on a 512k Mac.  It does not load the
;      Appletalk routines if there's only 128k of memory.
;  2)  There is very little error trapping.  It blithly assumes that most
;      Appletalk calls work well.
;  3)  The only way to remove a name from the list of talkers is by getting
;      a "goodbye" message.  It will get confused if someone's computer is
;      powered off or crashes without quitting the program.
;  4)  The strings used ("Walking in", "Walking out") are part of the
;      source code, not resources that could be edited.
;
Include 	MacTraps.D	; Use System and ToolBox traps
Include 	ToolEquX.D	; Use ToolBox equates
include		SysEquX.D
include		QuickEquX.D
include		FSEqu.D
;----------------------------- Use of Registers ---------------------------
ModifyReg	EQU	D4 	; D4 holds modifier bits from GetNextEvent 
MenuReg		EQU	D5	; D5 holds menu ID from MenuSelect,MenuKey
MenuItemReg	EQU	D6	; D6 holds item ID from MenuSelect,MenuKey
;--------------------------------- EQUATES --------------------------------

; These are equates associated with the resources
; for the Window example.

AppleMenu 	EQU	1	; First item in MENU resource
  AboutItem	EQU	1	; First item in Apple menu

FileMenu	EQU	2	; Second item in MENU resource
  QuitItem 	EQU	1	; First item in File menu

EditMenu	EQU	3	; Third item in MENU resource
  UndoItem	EQU	1	; Items in Edit menu
  CutItem	EQU	3	; (Item 2 is a line)
  CopyItem	EQU	4
  PasteItem	EQU	5
  ClearItem	EQU	6

AboutDialog	EQU	1	; About dialog is DLOG resource #1
ButtonItem	EQU	1	; First item in DITL used by DLOG #1
ASample		EQU	1	; Sample Window is WIND resource #1
SBlockSize	equ	256	;size of block to use for names & other things

;------------------------------- Main Program -----------------------------

Start	

	BSR		InitManagers		; Initialize managers
	BSR		SetupMenu		; Build menus, draw menu bar
	BSR		SetupWindow		; Draw Editing Window
	BSR		SetupTextEdit		; Initialize TextEdit
;*********************************************************************
	BSR		SetupChatter		; Do our initialization
	Clr.W		Quitflag(A5)		; say we're not done yet!
;*********************************************************************
EventLoop					; MAIN PROGRAM LOOP

	_SystemTask				; Update Desk Accessories
	; PROCEDURE	TEIdle (hTE:TEHandle);
	MOVE.L		MyhTE(A5),-(SP)		; Get handle to text record
	_TEIdle					; blink cursor etc.
		
	; FUNCTION	GetNextEvent(eventMask: INTEGER; 
	;		VAR theEvent: EventRecord) : BOOLEAN
	CLR 		-(SP)			; Clear space for result
;*********************************************************************
	MOVE 		#$FFFF,-(SP)		; Allow all 16 events
;*********************************************************************
	PEA		EventRecord		; Place to return results
	_GetNextEvent				; Look for an event
	MOVE		(SP)+,D0		; Get result code
	BEQ  		EventLoop		; No event... Keep waiting
	BSR 		HandleEvent		; Go handle event
;*********************************************************************
;  Since there are tasks to accomplish after the user chooses quit,
;  I have had to use a quitflag to mark when the program was finished.
;
	move.W		Quitflag(A5),D0		; are we done?
	Cmp.W		#0,D0
	BEQ		EventLoop		; Not Quit, keep going
	move.W		Broadcastflag(A5),D0	; still saying goodbye to everyone?
	Cmp.W		#0,D0
	BNE		EventLoop
	BSR		ClearEntity		; remove ourselves from ATalk
	LEA		ATPParamBlock(A5),A0	;close our socket
	move.W		#-11,ioRefNum(A0)	;use .ATP
	Clr.L		ioCompletion(A0)
	move.W		#250,csCode(A0)		;we're doing CloseATPskt
	move.b		MySocket(A5),+28(A0)	;close our socket
	_Control	,ASYNC
@0	move.W		ioResult(A0),D0		;wait for done
	Cmp.W		#0,D0
	BGT.S		@0
	_ExittoShell				; Quit, exit to Finder
;*********************************************************************

;------------------------------- InitManagers -----------------------------

InitManagers

	PEA		-4(A5)			; Quickdraw's global area
	_InitGraf				; Init Quickdraw
	_InitFonts				; Init Font Manager
	MOVE.L		#$0000FFFF,D0		; Flush all events
	_FlushEvents
	_InitWindows				; Init Window Manager
	_InitMenus				; Init Menu Manager
	CLR.L	-(SP)				; No restart procedure
	_InitDialogs				; Init Dialog Manager
	_TEInit					; Init Text Edit
	_InitCursor				; Turn on arrow cursor
	RTS

;------------------------------- SetupMenu ------------------------------

SetupMenu

; The names of all the menus and the commands in the menus are stored in the
; resource file.  The way you build a menu for an application is by reading
; each menu in from the resource file and then inserting it into the current
; menu bar.  Desk accessories are read from the system resource file and
; added to the Apple menu.

; Apple Menu Set Up.  

	; FUNCTION	GetMenu (menu ID:INTEGER): MenuHandle;
	CLR.L		-(SP)			; Space for menu handle
	MOVE 		#AppleMenu,-(SP)	; Apple menu resource ID 
	_GetRMenu				; Get menu handle 
	MOVE.L		(SP),MyAppleMenu(A5)	; Save for later comparison
	MOVE.L		(SP),-(SP)		; Copy handle for AddResMenu
	
	; PROCEDURE	InsertMenu (menu:MenuHandle; beforeID: INTEGER);
	CLR 		-(SP)			; Append to menu
	_InsertMenu				; Which is currently empty

; Add Desk Accessories Into Apple menu (Apple menu handle already on stack)

	; PROCEDURE	AddResMenu (menu: MenuHandle; theType: ResType);
	MOVE.L		#'DRVR',-(SP)		; Load all drivers 
	_AddResMenu				; And add to Apple menu

; File Menu Set Up

	; FUNCTION	GetMenu (menu ID:INTEGER): MenuHandle;
	CLR.L		-(SP)			; Space for menu handle
	MOVE 		#FileMenu,-(SP)		; File Menu Resource ID 
	_GetRMenu				; Get File menu handle
	Move.L		(SP),MyFileMenu(A5)	; save for later

	; PROCEDURE	InsertMenu (menu:MenuHandle; beforeID: INTEGER);
	CLR 		-(SP)			; Append to list
	_InsertMenu				; After Apple menu

; Edit Menu Set Up

	; FUNCTION	GetMenu (menu ID:INTEGER): MenuHandle;
	CLR.L		-(SP)			; Space for menu handle
	MOVE 		#EditMenu,-(SP)		; Edit menu resource ID
	_GetRMenu				; Get handle to menu
	MOVE.L		(SP),MyEditMenu(A5)	; Save for later
					        ; Leave on stack for Insert
	; PROCEDURE	InsertMenu (menu:MenuHandle; beforeID: INTEGER);
	CLR 		-(SP)			; Append to list
	_InsertMenu				; After File menu
	_DrawMenuBar				; Display the menu bar
	RTS
	
;------------------------------ SetupWindow ------------------------------

SetupWindow

; The window parameters are stored in our resource file.  Read them from 
; the file and draw the window, then set the port to that window.  Note that
; the window parameters could just as easily have been set using the call
; NewWindow, which doesn't use the resource file.

	; FUNCTION	GetNewWindow (windowID: INTEGER; wStorage: Ptr; 
	;			      behind: WindowPtr) : WindowPtr;
	CLR.L		-(SP)			; Space for window pointer
	MOVE 		#ASample,-(SP)		; Resource ID for window
	PEA		WindowStorage(A5)	; Storage for window
	MOVE.L		#-1,-(SP)		; Make it the top window
	_GetNewWindow				; Draw the window
	MOVE.L		(SP),MyWindow(A5)	; Save for later 
	
	; PROCEDURE	SetPort (gp: GrafPort)	; Pointer still on stack
	_SetPort				; Make it the current port
	
	RTS
	
;---------------------------- SetupTextEdit --------------------------

SetupTextEdit

; Create a new text record for TextEdit, and define the window within which
; it will be displayed.  Note that if the window boundaries are changed in
; the resource file, DestRect and ViewRect will have to be changed too.

	; PROCEDURE	TENew (destRect,viewRect: Rect): TEHandle;
	CLR.L		-(SP)			; Space for text handle 
	PEA		DestRect		; DestRect Rectangle
	PEA		DestRect		; ViewRect Rectangle
	_TENew					; New Text Record
;*********************************************************************
;  Chatter requires two text areas--one for the "display screen", and
;  one for the entry of messages.  It also needs a button for the user
;  to press to transmit the message
;
	MOVE.L		(SP)+,MyhTE(A5)		; Save text handle
	CLR.L		-(SP)			; Space for text handle 
	PEA		Dest2		; DestRect Rectangle
	PEA		Dest2		; ViewRect Rectangle
	_TENew					; New Text Record
	MOVE.L		(SP)+,OutputhTE(A5)		; Save text handle
	Clr.L		-(SP)			;get the transmit button
	move.W		#1,-(SP)
	move.L		MyWindow(A5),-(SP)
	_GetNewControl
	move.L		(SP)+,MyButton(A5)
;*********************************************************************
	RTS
	
;*********************************************************************
;---------------------------- SetupChatter --------------------------
;
; This is the Appletalk initialization that was needed.  We must
;  (a) start the lookup of other chatters already on the network
;  (b) get a socket
;  (c) register our name onto the network
;  (d) ask for our first "request" (message) from the network
;  (e) and initialize some variables
;
SetupChatter
	Clr.W	MyEntFlag(A5)		;mark not yet registered
	
	BSR	StartWho		;start a lookup.  This will post
					;a event#13 when it's finished
	move.W	#$FFFF,NeedHello(A5)	;mark to send out our walking in
					;messages when the lookup is done.
					
	LEA	ATPParamBlock(A5),A0	;get ourselves a socket
	move.W	#-11,ioRefNum(A0)	;use .ATP
	Clr.L	ioCompletion(A0)
	move.W	#254,csCode(A0)		;we're doing OpenATPskt
	Clr.w	+28(A0)			;say we'll take any socket
	Clr.L	+30(A0)			;accept calls from anybody
	_Control	,ASYNC
@0	move.W	ioResult(A0),D0		;wait for done
	Cmp.W	#0,D0
	BGT.S	@0
	BLT	OopsReturn		;if error, clean up & quit
	move.B	+28(A0),MySocket(A5)	;save our socket number
;
; Now, let's get the user's name
;
	CLR.L	-(SP)			; Get our Dialog
	move	#2,-(SP)
	PEA	DStorage
	Move.L	#-1,-(SP)
	_GetNewDialog
	move.L	(SP),-(SP)		;push the pointer down again
	move.W	#4,-(SP)
	Clr.W	-(SP)
	move.W	#$7FFF,-(SP)
	_SelIText
	Clr.L	-(SP)			;handle it
	PEA	ItemHit
	_ModalDialog
	move.W	Itemhit,D0		;did they choose cancel?
	Cmp.W	#1,D0
	BNE	OopsReturn		;if they cancelled, quit!
	move.L	(SP),-(SP)		;get the new name
	move.W	#4,-(SP)
	PEA	TempType
	PEA	TempItem
	PEA	TempBox
	_GetDItem
	move.L	TempItem,-(SP)
	PEA	MyEntity+9(A5)
	_GetIText
	_CloseDialog
	move.B	MyEntity+9(A5),D0	;was it null?
	cmp.B	#0,D0
	BEQ	OopsReturn
	Cmp.B	#15,D0			;16 or more characters?
	BLE.S	@50
	move.B	#15,D0			;if so, force to 15
	move.B	D0,MyEntity+9(A5)
@50	Add.B	#1,D0
	AndI.L	#$FF,D0
	LEA	MyEntity+9(A5),A1	;add on the network & type names
	Add.L	D0,A1
	LEA	ChatType,A0
	move.L	#10,D0
	_BlockMove
	Clr.w	MyEntity+4(A5)		;local network,Register appears to set the nodeID
	move.B	MySocket(A5),MyEntity+7(A5)	;our socket
	LEA	ATPParamBlock(A5),A0	;Register our name
	move.W	#-10,ioRefNum(A0)	;use .MPP
	Clr.L	ioCompletion(A0)
	move.W	#253,csCode(A0)		;do a register name
	LEA	MyEntity(A5),A1
	move.B	#2,+28(A0)		;retry every 10 ticks
	move.B	#2,+29(A0)		;retry 6 times
	move.L	A1,+30(A0)
	move.b	#$FF,+34(A0)		;do a verify
	_Control	,ASYNC		;start the register up
@100	move.W	ioResult(A0),D0
	Cmp.W	#0,D0
	BGT.S	@100
	move.W	#$FFFF,MyEntFlag(A5)	;mark we've got a name registered
	BSR	MyGetRequest		;Prepare to get our first request
					;An event#12 will be posted when
					;it comes in.
	Clr.W	Broadcastflag(A5)	;say we aren't broadcasting
	RTS

OopsReturn
	LEA	NBPParamBlock(A5),A0	;make sure all calls are done
@0	move.W	ioResult(A0),D0		;wait for done
	Cmp.W	#0,D0
	BGT.S	@0
	LEA	ATPParamBlock(A5),A0
@1	move.W	ioResult(A0),D0		;wait for done
	Cmp.W	#0,D0
	BGT.S	@1
	BSR	ClearEntity		; remove ourselves from ATalk
	_ExittoShell
;*********************************************************************
;------------------------- Event Handling Routines -----------------------

HandleEvent

; Use the event number as an index into the Event table.  These 16 events
; are all the things that could spontaneously happen while the program is
; in the main loop.

	MOVE		Modify,ModifyReg	; More useful in a reg
	MOVE		What,D0			; Get event number
	ADD		D0,D0			; *2 for table index
	MOVE		EventTable(D0),D0	; Point to routine offset
	JMP		EventTable(D0)		; and jump to it

EventTable	

	DC.W		NextEvent-EventTable	; Null Event (Not used)
	DC.W		MouseDown-EventTable	; Mouse Down
	DC.W		NextEvent-EventTable	; Mouse Up (Not used)
	DC.W		KeyDown-EventTable	; Key Down
	DC.W		NextEvent-EventTable	; Key Up (Not used)
	DC.W		KeyDown-EventTable	; Auto Key
	DC.W		Update-EventTable	; Update 
	DC.W		NextEvent-EventTable	; Disk (Not used) 
	DC.W		Activate-EventTable	; Activate
	DC.W		NextEvent-EventTable	; Abort (Not used)
	DC.W		NextEvent-EventTable	; Network (Not used)
	DC.W		NextEvent-EventTable	; I/O Event (Not used)
;*********************************************************************
	DC.W		MyGotRequest-EventTable	; Application-1 (GotRequest)
	DC.W		EndWho-EventTable	; Application-2 (Lookup finished)
	DC.W		EndMess-EventTable	; Application-3 (SendRequest finished)
	DC.W		EndResp-EventTable	; Application-4 (Not used)
;*********************************************************************
	
;--------------------------- Event Actions ---------------------------

Activate		

; An activate event is posted by the system when a window needs to be
; activated or deactivated.  The information that indicates which window
; needs to be updated was returned by the NextEvent call.

	Move.L		MyWindow(A5),D0		; is this our window?
	CMP.L		Message,D0		; Was it our window?
	BNE 		NextEvent		; No, get next event
	BTST		#0,ModifyReg 		; Activate?
	BEQ 		Deactivate		; No, go do Deactivate

; To activate our window, activate TextEdit, and disable Undo since we don't
; support it.  Then set our window as the port since an accessory may have
; changed it.  This activate event was generated by SelectWindow as a result
; of a click in the content region of our window.  If the window had scroll
; bars, we would do ShowControl and HideControl here too.

	; PROCEDURE	TEActivate (hTE: TEHandle);
	MOVE.L		MyhTE(A5),-(SP)		; Move Text Handle To Stack
	_TEActivate				; Activate Text

	; PROCEDURE DisableItem (menu:MenuHandle; item:INTEGER);
	MOVE.L	MyEditMenu(A5),-(SP)		; Get handle to the menu
	MOVE	#UndoItem,-(SP)			; Enable 1st item (undo)
	_DisableItem

SetOurPort					; used by InAppleMenu

	; PROCEDURE 	SetPort (gp: GraphPort) ; Set the port to us, since
	MOVE.L		MyWindow(A5),-(SP)	; an accessory might have
	_SetPort				; changed it.

NextEvent	

	MOVEQ #0,D0				; Say that it's not Quit
	RTS					; return to EventLoop
		
Deactivate

; To deactivate our window, turn off TextEdit, and Enable undo for the desk
; accessories (which must be active instead of us).

	; PROCEDURE	TEDeActivate (hTE: TEHandle)
	MOVE.L		MyhTE(A5),-(SP)		; Get Text Handle
	_TeDeActivate				; Un Activate Text

	; PROCEDURE EnableItem (menu:MenuHandle; item:INTEGER);
	MOVE.L		MyEditMenu(A5),-(SP)	; Get handle to the menu
	MOVE		#UndoItem,-(SP)		; Enable 1st item (undo)
	_EnableItem
	BRA 		NextEvent		; Go get next event
	
Update	
;*********************************************************************
; Quite a few changes here.  We needed to
;  (a) Draw the transmit button
;  (b) Draw the teRec that represents our screen
;  (c) Draw the teRec with our outgoing message in it (frame this teRec)
;  (d) Draw the list of chatters on the system.
;
	movem.L		D3-D7/A2-A4,-(SP)	
	MOVE.L		MyWindow(A5),-(SP)	; Get pointer to window
	_BeginUpDate				; Begin the update
	move.L		MyWindow(A5),-(SP)	;draw the controls
	_DrawControls
	PEA		ViewRect		; Erase visible area
	_EraseRect
	PEA		View2
	_EraseRect
	PEA		ViewRect
	_FrameRect
	
	PEA		DestRect		; Get visible area
	MOVE.L		MyhTE(A5),-(SP)		; and handle to text 
	_TEUpdate				; then update the window
	PEA		View2
	move.L		OutputhTE(A5),-(SP)
	_TEUpdate
	PEA		NameRect		;draw the names area
	_EraseRect
	move.W	NameRect+left,-(SP)		;draw the line
	move.W	NameRect+top,-(SP)
	_Moveto
	move.W	NameRect+left,-(SP)
	move.W	NameRect+bottom,-(SP)
	_Lineto
	Clr.L	D3
	move.w	Namerect+top,D4			;setup for starting to give names
	Add.W	#20,D4
	LEA	MyNames(A5),A4			;where's the names?
@0	Cmp.W	MyNameCount(A5),D3		;all done?
	BGE	@20
	move.W	NameRect+left,D0		;move to position
	AddQ	#3,D0
	move.W	D0,-(SP)
	move.W	D4,-(SP)
	_Moveto
	PEA	+5(A4)				;print the name!
	_DrawString
;
; This code, normally left out, can be inserted back in if you
; want the list of names to include the nodeID & socket.  This can
; be useful while experimenting or debugging
;
		if	0
		move.W	#'(',-(SP)
		_DrawChar
		move.B	+2(A4),D0		;print the numbers
		AndI.L	#$FF,D0			;convert to a string
		LEA	Tempstr(A5),A0
		Clr.W	-(SP)
		_Pack7
		PEA	Tempstr(A5)
		_DrawString
		move.W	#',',-(SP)
		_DrawChar
		move.B	+3(A4),D0
		AndI.L	#$FF,D0			;convert to a string
		LEA	Tempstr(A5),A0
		Clr.W	-(SP)
		_Pack7
		PEA	Tempstr(A5)
		_DrawString
		move.W	#')',-(SP)
		_DrawChar
		endif
		
		Add.W	#20,D4			;point to next line down
		Add.L	#22,A4			;point to next name
		AddQ	#1,D3			;increment our count by one
		BRA	@0
@20

	; PROCEDURE	EndUpdate (theWindow: WindowPtr);
	MOVE.L		MyWindow(A5),-(SP)	; Get pointer to window
	_EndUpdate				; and end the update
	movem.L		(SP)+,D3-D7/A2-A4
	BRA 		NextEvent		; Go get next event
;*********************************************************************


;*********************************************************************
; We needed to treat special keys (return, tab, enter) as presses of the
; transmit button (if it's active).

KeyDown	

; A key was pressed.  First check to see if it was a command key.  If so, 
; go do it.  Otherwise pass the key to TextEdit.

	BTST		#8,ModifyReg	; Is command key down?
	BNE 		CommandDown		; If so, handle command key

	move		Message+2,D0		;what's the key?
	AndI.W		#$FF,D0
	Cmp.W		#13,D0			;Return?
	BEQ		DoCR
	Cmp.B		#3,D0			;Enter?
	BEQ		DoCR
	Cmp.B		#9,D0			;Tab?
	BEQ		DoCR
	; PROCEDURE	TEKey (key: CHAR; hTE: TEHandle);
	MOVE 		Message+2,-(SP)		; Get character
	MOVE.L		MyhTE(A5),-(SP)		; and text record
	_TEKey					; Give char to TextEdit
	BRA 		NextEvent		; Go get next event
;
; On a special key, see if the transmit key is active.  If so, pretend to
; press it, and transmit what's in the message buffer.
;
DoCR	move.W		MyActiveFlag(A5),D0	;is it active?
	Cmp.W		#0,D0
	BEQ		NextEvent
	BRA		ShipandSend		;if so, transmit away!
;*********************************************************************
	
CommandDown	

; The command key was down.  Call MenuKey to find out if it was the command 
; key equivalent for a menu command, pass the menu and item numbers to Choices.

	; FUNCTION	MenuKey (ch:CHAR): LongInt;
	CLR.L		-(SP)			; Space for Menu and Item
	MOVE 		Message+2,-(SP)		; Get character
	_MenuKey				; See if it's a command
	MOVE		(SP)+,MenuReg		; Save Menu
	MOVE		(SP)+,MenuItemReg	; and Menu Item
	BRA 		Choices			; Go dispatch command


;--------------------Mouse Down Events And Their Actions----------------------

MouseDown	

; If the mouse button was pressed, we must determine where the click
; occurred before we can do anything.  Call FindWindow to determine
; where the click was; dispatch the event according to the result.

	; FUNCTION	FindWindow (thePt: Point; 
	;		            VAR whichWindow: WindowPtr): INTEGER;
	CLR 		-(SP)			; Space for result
	MOVE.L		Point,-(SP)		; Get mouse coordinates
	PEA		WWindow			; Event Window
	_FindWindow				; Who's got the click? 
	MOVE 		(SP)+,D0		; Get region number
	ADD		D0,D0			; *2 for index into table
	MOVE		WindowTable(D0),D0	; Point to routine offset
	JMP		WindowTable(D0)		; Jump to routine
	
WindowTable

	DC.W		NextEvent-WindowTable	; In Desk (Not used)
	DC.W		InMenu-WindowTable	; In Menu Bar
	DC.W		SystemEvent-WindowTable ; System Window
	DC.W		Content-WindowTable	; In Content
	DC.W		Drag-WindowTable	; In Drag
	DC.W		NextEvent-WindowTable	; In Grow (Not used)
	DC.W		NextEvent-WindowTable	; In Go Away (Not used)

SystemEvent	

; The mouse button was pressed in a system window.  SystemClick calls the 
; appropriate desk accessory to handle the event.

	; PROCEDURE	SystemClick (theEvent: EventRecord;
	;		             theWindow: WindowPtr);
	PEA		EventRecord		; Get event record
	MOVE.L		WWindow,-(SP)		; and window pointer
	_SystemClick				; Let the system do it
	BRA 		NextEvent		; Go get next event
	
Content	

; The click was in the content area of a window.  If our window was in 
; front, then call Quickdraw to get local coordinates, then pass the 
; coordinates to TextEdit. We also determine whether the shift key was 
; pressed so TextEdit can do shift-clicking. If our window wasn't in 
; front, move it to the front, but don't process click.

	CLR.L		-(SP)			; clear room for result
	_FrontWindow				; get FrontWindow
	MOVE.L		(SP)+,D0		; Is front window pointer
	CMP.L		MyWindow(A5),D0		; same as our pointer?
	BEQ.S 	@1				; Yes, call TextEdit

; We weren't active, select our window.  This causes an activate event.

	; PROCEDURE	SelectWindow (theWindow: WindowPtr);
	MOVE.L		WWindow,-(SP)		; Window Pointer To Stack
	_SelectWindow				; Select Window
	BRA 		NextEvent		; and get next event

@1	

; We were active, pass the click (with shift) to TextEdit.

	; PROCEDURE	GlobalToLocal (VAR pt:Point);
	PEA		Point			; Mouse Point
	_GlobalToLocal				; Global To Local
;*********************************************************************
;  First, we must see if the user pressed in the transmit button.
;  Then, since now only a part of the window is in the editable teRec,
;  we must see if the user pressed within that section.
;
	Clr.W	-(SP)			;are we in a control?
	move.L	Point,-(SP)
	move.L	MyWindow(A5),-(SP)
	PEA	Tempstr(A5)
	_FindControl
	move.W	(SP)+,D0
	Cmp.W	#0,D0
	BNE	InaControl
	
	Clr.W	-(SP)			;are we in the text area?
	move.L	Point,-(SP)
	PEA	ViewRect
	_PtinRect
	move.W	(SP)+,D0
	Cmp.W	#0,D0
	BEQ	NextEvent
;*********************************************************************

	; PROCEDURE 	TEClick (pt: Point; extend: BOOLEAN; hTE: TEHandle);
	MOVE.L		Point,-(SP)		; Mouse Point (GTL)
	BTST		#9,ModifyReg	; Is shift key down?
	SNE		D0			; True if shift down
	
; Note:  We want the boolean in the high byte, so use MOVE.B.  The 68000
; pushes an extra, unused byte on the stack for us.

	MOVE.B		D0,-(SP)		
	MOVE.L		MyhTE(A5),-(SP)		; Identify Text
	_TEClick				; TEClick
	BRA 		NextEvent		; Go get next event
;*********************************************************************
;
; We're in a control--our push button
;  See if the user released within it.  If so, transmit away!
;
InaControl
	Clr.W	-(SP)			;track it
	move.L	MyButton(A5),-(SP)
	move.L	Point,-(SP)
	Clr.L	-(SP)
	_TrackControl
	move.W	(SP)+,D0		;was it selected?
	Cmp.W	#0,D0
	BNE	ShipandSend		;if so, send the message & clear it
	BEQ	NextEvent
;*********************************************************************
	
Drag

; The click was in the drag bar of the window.  Draggit.

	; DragWindow (theWindow:WindowPtr; startPt: Point; boundsRect: Rect);
	MOVE.L		WWindow,-(SP)		; Pass window pointer
	MOVE.L		Point,-(SP)		; mouse coordinates
	PEA		MyBounds			; and boundaries
	_DragWindow				; Drag Window
	BRA 		NextEvent		; Go get next event
	
InMenu	

; The click was in the menu bar.  Determine which menu was selected, then
; call the appropriate routine.

	; FUNCTION	MenuSelect (startPt:Point) : LongInt;
	CLR.L		-(SP)			; Get Space For Menu Choice
	MOVE.L		Point,-(SP)		; Mouse At Time Of Event
	_MenuSelect				; Menu Select
	MOVE 		(SP)+,MenuReg		; Save Menu
	MOVE		(SP)+,MenuItemReg	; and Menu Item

; On entry to Choices, the resource ID of the Menu is saved in the low
; word of a register, and the resource ID of the MenuItem in another.
; The routine MenuKey, used when a command key is pressed, returns the same 
; info.

Choices						; Called by command key too

	CMP 		#AppleMenu,MenuReg	; Is It In Apple Menu?
	BEQ 		InAppleMenu		; Go do Apple Menu
	CMP 		#FileMenu,MenuReg	; Is It In File Menu?
	BEQ 		InFileMenu		; Go do File Menu
	CMP 		#EditMenu,MenuReg	; Is It In Edit Menu?
	BEQ 		InEditMenu		; Go do Edit Menu
	
ChoiceReturn

	BSR		UnHiliteMenu		; Unhighlight the menu bar
	BRA 		NextEvent		; Go get next event
		
InFileMenu

; If it was in the File menu, just check for Quit since that's all there is.

	CMP	#QuitItem,MenuItemReg	; Is It Quit?
	BNE 	ChoiceReturn		; No, Go get next event
	BSR	UnHiliteMenu		; Unhighlight the menu bar
;*********************************************************************
;  When we quit, we must now send a message to everyone saying we've left
;  the room.
;
	move.W	#-1,Quitflag(A5)	; say we've quit!
	move.W	MyNameCount(A5),D0	;is there anyone to say goodbye to?
	Cmp.W	#0,D0
	BEQ	@100
	LEA	MyEntity+9(A5),A0	;copy our name over
	LEA	MyMessageBlock(A5),A1
	move.L	#16,D0
	_BlockMove
	BSR	SetupMessage		;prepare to send a message
	move.W	#3,+18(A0)		;say this is a goodbye
	BSR	StartBroadcast
@100	RTS
;*********************************************************************
	
InEditMenu

; First, call SystemEdit.  If a desk accessory is active that uses the Edit
; menu (such as the Notepad) this lets it use our menu.
; Decide whether it was cut, copy, paste, or clear.  Ignore Undo since we
; didn't implement it.

	BSR		SystemEdit		; Desk accessory active?
	BNE.S 		ChoiceReturn		; Yes, SystemEdit handled it
	CMP		#CutItem,MenuItemReg	; Is It Cut?
	BEQ 		Cut			; Yes, go handle it
	CMP		#CopyItem,MenuItemReg	; Is it Copy?
	BEQ 		Copy			; Yes, go handle it
	CMP		#PasteItem,MenuItemReg	; Is it Paste?
	BEQ 		Paste			; Yes, go handle it
	CMP		#ClearItem,MenuItemReg	; Is it Clear?
	BEQ 		ClearIt			; Yes, go handle it
	BRA 		ChoiceReturn		; Go get next event
	
InAppleMenu

; It was in the Apple menu.  If it wasn't About, then it must have been a 
; desk accessory.  If so, open the desk accessory.  

	CMP		#AboutItem,MenuItemReg	; Is It About?
	BEQ 		About			; If So Goto About...
	
	; PROCEDURE	GetItem (menu: MenuHandle; item: INTEGER; 
	;                        VAR itemString: Str255);
	MOVE.L		MyAppleMenu(A5),-(SP)	; Look in Apple Menu
	MOVE 		MenuItemReg,-(SP)	; What Item Number?
	PEA		DeskName		; Get Item Name
	_GetItem				; Get Item

	; FUNCTION	OpenDeskAcc (theAcc: Str255) : INTEGER;
	CLR 		-(SP)			; Space For Opening Result
	PEA		DeskName		; Open Desk Acc
	_OpenDeskAcc				; Open It
	MOVE 		(SP)+,D0		; Pop result
	
GoSetOurPort

	BSR 		SetOurPort		; Set port to us 
	BRA		ChoiceReturn		; Unhilite menu and return
						
;--------------------------- Text Editing Routines ------------------------
;*********************************************************************
; No changes here, just a note to point out that these do NOT cut and
; paste to the normal Macintosh clipboard.  For a real, production product,
; this would have to be changed
;*********************************************************************

Cut						; CUT

	; PROCEDURE	TECut (hTE: TEHandle);
	MOVE.L		MyhTE(A5),-(SP)		; Identify Text
	_TECut					; Cut it and copy it
	BRA 		ChoiceReturn		; Go get next event

Copy						; COPY

	; PROCEDURE	TECopy (hTE: TEHandle);
	MOVE.L		MyhTE(A5),-(SP)		; Identify Text
	_TECopy					; Copy text to clipboard
	BRA 		ChoiceReturn		; Go get next event

Paste						; PASTE

	; PROCEDURE	TEPaste (hTE: TEHandle);
	MOVE.L	MyhTE(A5),-(SP)			; Identify Text
	_TEPaste				; Paste
	BRA 	ChoiceReturn			; Go get next event

ClearIt						;CLEAR

	; PROCEDURE	TEDelete (hTE: TEHandle);
	MOVE.L		MyhTE(A5),-(SP)		; Point to text
	_TEDelete				; Clear without copying
	BRA 		ChoiceReturn		; Go get next event

; SystemEdit does undo, cut, copy, paste, and clear for desk accessories.
; It returns False (BEQ) if the active window doesn't belong to a
; desk accessory.

SystemEdit
	
	; FUNCTION	SystemEdit (editCmd:INTEGER): BOOLEAN;
	CLR 		-(SP)			; Space for result
	MOVE		MenuItemReg,-(SP)	; Get item in Edit menu
	SUBQ		#1,(SP)			; SystemEdit is off by 1
	_SysEdit				; Do It 
	MOVE.B 	(SP)+,D0			; Pop result
	RTS 					; BEQ if NOT handled

UnhiliteMenu
	
	; PROCEDURE	HiLiteMenu (menuID: INTEGER);
	CLR		-(SP)			; All Menus
	_HiLiteMenu				; UnHilite Them All
	RTS
	

;---------------------------------Misc Routines---------------------------

About

; Call GetNewDialog to read the dialog box parameters from the resource file
; and display the box.  Set the port to the box, then wait for the proper 
; click or keypress.  Finally, close the dialog box and set the pointer to us.

	; FUNCTION	GetNewDialog (dialogID: INTEGER; dStorage: Ptr; 
	;                             behind: WindowPtr) : DialogPtr
	CLR.L		-(SP)			; Space For dialog pointer
	MOVE 		#AboutDialog,-(SP)	; Identify dialog rsrc #
	PEA		DStorage		; Storage area
	MOVE.L		#-1,-(SP)		; Dialog goes on top
	_GetNewDialog				; Display dialog box
	MOVE.L		(SP),-(SP)		; Copy handle for Close

	; PROCEDURE	SetPort (gp: GrafPort)	; Handle already on stack
	_SetPort				; Make dialog box the port

	; PROCEDURE	TEDeActivate (hTE: TEHandle)
	MOVE.L		MyhTE(A5),-(SP)		; Identify Text
	_TEDeActivate				; Deactivate Text

WaitOK

	; PROCEDURE	ModalDialog (filterProc: ProcPtr; 
	;			     VAR itemHit: INTEGER);
	CLR.L		-(SP)			; Clear space For handle
	PEA		ItemHit			; Storage for item hit
	_ModalDialog				; Wait for a response

	MOVE		ItemHit,D0		; Look to see what was hit
	CMP		#ButtonItem,D0		; was it OK?
	BNE  		WaitOK			; No, wait for OK
	
	; PROCEDURE	CloseDialog (theDialog: DialogPtr);
	_CloseDialog				; Handle already on stack
	 BRA		GoSetOurPort		; Set port to us and return
;*********************************************************************
;  All the code from here and below is the special Appletalk code written
;  specifically for Chatter.
;
; ShipandSend is called when either the transmit button is pressed, or a
; tab/return/enter key is pressed.  It will
;  (a) convert the text in myhTE into a string, storing it in MyMessageBlock
;  (b) clear out myhTE, so the user can type in the next message
;  (c) start a broadcast of the message.
;
ShipandSend
	move.L	myhTE(A5),A0		;copy the text into MyMessageBlock
	move.L	(A0),A0			;for later sending
	move.L	teTextH(A0),-(SP)	;cute trick to convert to string
	PEA	MyMessageBlock(A5)
	_GetIText
	Clr.L	-(SP)			;clear the text buffer
	move.L	#$7FFFF,-(SP)
	move.L	myhTE(A5),-(SP)
	_TESetSelect
	move.L	myhTE(A5),-(SP)
	_TEDelete
	move.B	MyMessageBlock(A5),D0	;was there anything in the buffer?
	cmp.B	#0,D0
	BEQ	NextEvent
	move.W	MyNameCount(A5),D0	;do we have anybody to send to?
	Cmp.W	#0,D0
	BEQ	NextEvent
	BSR	SetupMessage		;prepare to send a message
	move.W	#2,+18(A0)		;say this is a message
	BSR	StartBroadcast
	BRA	NextEvent
;
; This sets up the parameter block for sending a request.
;   It assumes that MyMessageBlock has the string to send as part of the
;   request.  It does NOT set the user data, this is set in the routine
;   that calls this to allow us to send several types of requests through
;   this one message
;
SetupMessage	BSR	DisableATalk
		LEA	ATPParamBlock(A5),A0	;set up for a call to ATP
		move.W	#-11,ioRefNum(A0)	;use .MPP
		LEA	MessComplet,A1		;call who completion when we're done
		move.L	A1,ioCompletion(A0)
		move.W	#255,csCode(A0)		;we're doing a SendRequest
		move.B	#1,+29(A0)		;try more than once
		LEA	MyMessageBlock(A5),A1	;the request data is in MyMessageBlock
		move.L	A1,+36(A0)
		move.B	(A1),D0			;get the length
		AndI.W	#$FF,D0
		AddQ	#1,D0
		move.W	D0,+34(A0)		;save the length
		LEA	InBDS(A5),A1		;setup the BDS's
		move.L	A1,+40(A0)
		move.B	#1,+44(A0)		;one responses expected
		move.B	#3,+45(A0)		;quick timeouts
		move.B	#2,+47(A0)		;only try once
		LEA	InBDSBuffer(A5),A1	;setup the BDS, for responses
		move.L	A1,InBDS+2(A5)
		move.W	#256,InBDS(A5)
		RTS
;
; StartBroadcast--this will start a message being sent to all people on the list
;  It starts an asynchronous control call, with a completion routine that
;  will post an event#14 when it's finished.
;
; NextSend--this finds the next name in our list, and will start the request
;  to him (with the completion routine as described above)
;
StartBroadcast	move.W	#$FFFF,Broadcastflag(A5)	;this marks it as a broadcast
		move.W	#$FFFF,NextBroadcast(A5)	;trick to start at zero
NextSend	move.W	NextBroadcast(A5),D0	;get next number
		AddQ	#1,D0
		move.W	D0,NextBroadcast(A5)
		Cmp.W	MyNameCount(A5),D0	;are we all done?
		BGE	@999
		AndI.L	#$FFFF,D0		;no.  get the next address
		MulU	#22,D0
		LEA	MyNames(A5),A1
		LEA	ATPParamBlock(A5),A0
		move.L	+0(A1,D0.w),+30(A0)	;copy the address over
		move.B	#2,+47(A0)		;only try once
		_Control	,ASYNC
		RTS
@999		Clr.W	Broadcastflag(A5)	;mark we're done
		Clr.W	NextBroadcast(A5)
		RTS
;
; This routine is called when a request (message) is finished.
;  It is called as a result of an event#14, posted by the message
;  completion routine.
;  It will call NextSend to send to the next person, if you're
;  broadcasting to everyone.
;
EndMess		move.W	Broadcastflag(A5),D0	;are we in the middle of a broadcast?
		Cmp.W	#0,D0
		BEQ	@900
		BSR	NextSend		;send to the next
		move.W	Broadcastflag(A5),D0	;was there a next?
		Cmp.W	#0,D0
		BNE	@999
@900		BSR	EnableATalk		;reenable commands
@999		BRA	ChoiceReturn
;
; DoGetWho cannot be called in the current version of chatter.  An earlier
; version allowed the user to manually start a new lookup of everyone on
; the network, from a menu selection.
;
DoGetWho	BSR	StartWho
		BRA	ChoiceReturn
;
; StartWho sets up the NBPParamBlock to do a lookupname transaction.  This
; will be completed asynchronously, and the completion routine will post
; and event#13 when it is finished.  In the current version it is only
; called during initialization, but you may wish to call it elsewhere
; to take care of people powering down without first quitting the program.
;
StartWho	BSR	DisableATalk
		LEA	NBPParamBlock(A5),A0	;set up for a call to NBP
		move.W	#-10,ioRefNum(A0)	;use .MPP
		LEA	WhoComplet,A1		;call who completion when we're done
		move.L	A1,ioCompletion(A0)
		move.W	#251,csCode(A0)		;we're doing a LookupName
		move.B	#2,+28(A0)		;retry every 2 seconds
		move.B	#2,+29(A0)		;retry 2 times
		LEA	SearchEntity,A1		;look for everything
		move.L	A1,+30(A0)
		LEA	MySearchBlock(A5),A1	;and say where to put it
		move.L	A1,+34(A0)
		move.w	#SBlockSize,+38(A0)	;buffersize
		move.W	#10,+40(A0)		;maximum number to get
		Clr.W	+42(A0)			;say we got zero.
		_Control	,ASYNC
		RTS
;
; This routine is called as a result of the Lookup finishing
;  (event#13, posted by the Lookup completion routine).  It copies the
;  results over into our own array of addresses and names.  It then will
;  see if this is part of the initialization.  If we're just getting
;  started, it will send a "hello" message to eveyone.
;
EndWho		move.W	NBPParamBlock+ioResult(A5),D0	;did it go okay?
		Cmp.W	#0,D0
		BLT	ChoiceReturn
		movem.L	D2-D5/A3-A4,-(SP)	;save the A4 register
		Clr.W	D2			;get our count started
		move.W	NBPParamBlock+42(A5),D3	;find out how many to print
		move.W	D3,MyNameCount(A5)	;and save the count!
		LEA	MySearchBlock(A5),A4	;and point A4 at the data
		LEA	MyNames(A5),A3
@0		Cmp.W	D2,D3			;are we done?
		BLE	@900
		Clr.L	D0			;copy over 22 bytes (name+address)
@1		move.b	+0(A4,D0.L),+0(A3,D0.L)
		AddQ	#1,D0
		Cmp.W	#22,D0
		BLT.S	@1
		Add.L	#22,A3			;point to next entry in table
		AddQ	#5,A4			;point at the name.
		BSR	PointtoNext		;point past strings
		BSR	PointtoNext
		BSR	PointtoNext
		AddQ	#1,D2			;and next item!
		BRA	@0
@900		movem.L	(SP)+,D2-D5/A3-A4
		
		move.L	MyWindow(A5),-(SP)	;redraw the names window
		_Setport
		PEA	NameRect
		_InvalRect
		Move.W	NeedHello(A5),D0	;do we need to say hello to everyone!
		Cmp.W	#0,D0
		BEQ	@100
		Clr.W	NeedHello(A5)		;yes.  start it up
		move.W	MyNameCount(A5),D0	;is there anyone to say hello to?
		Cmp.W	#0,D0
		BEQ	@100			;if not, we're done.
		LEA	MyEntity+9(A5),A0	;copy our name over
		LEA	MyMessageBlock(A5),A1
		move.L	#16,D0
		_BlockMove
		BSR	SetupMessage		;prepare to send a message
		move.W	#1,+18(A0)		;say this is a hello
		move.B	MySocket(A5),+20(A0)	;pass the socket here!
						;since the socket we're sending
						;the request on isn't the
						;socket we want answers to,
						;we must include the socket
						;as part of the user bytes.
		BSR	StartBroadcast		
		BRA	ChoiceReturn

@100		BSR	EnableATalk
		BRA	ChoiceReturn

PointtoNext	move.B	(A4),D0
		AndI.L	#$FF,D0
		ADDQ	#1,D0
		Add.L	D0,A4
		RTS
;
; This will unregister our name, if we've got one registered.
;
ClearEntity	move.W	MyEntFlag(A5),D0	;are we already registered?
		Cmp.W	#0,D0
		BEQ.S	@999
		LEA	NBPParamBlock(A5),A0	;yes!  set up for a call to NBP
		move.W	#-10,ioRefNum(A0)	;use .MPP
		Clr.L	ioCompletion(A0)
		move.W	#252,csCode(A0)		;do a remove name
		LEA	MyEntity(A5),A1		;remove our name
		move.L	A1,+30(A0)
		_Control	,ASYNC		;do the control call
@0		move.W	ioResult(A0),D0
		Cmp.W	#0,D0
		BGT.S	@0
@999		RTS

;
; This prepares chatter to receive a request (a message).  It's called
; asynchronously, and will post an event#12 when a request comes in.
;
MyGetRequest	LEA	RequestParamBlock(A5),A0	;setup the call to ATP
		move.W	#-11,ioRefNum(A0)
		LEA	RequestComplet,A1		;we've got a completion!
		move.L	A1,ioCompletion(A0)
		move.W	#253,cscode(A0)
		move.W	MySocket(A5),+28(A0)
		move.W	#SBlocksize,+34(A0)		;save the block size
		LEA	MyRequestBlock(A5),A1
		move.L	A1,+36(A0)
		_Control	,ASYNC		;start the control call out!
		RTS
;
; A request has been received.  It may be someone walking in, someone
;  walking out, or someone sending a message.
;
MyGotRequest	move.W	RequestParamBlock+18(A5),D0	;what was it?
		Cmp.W	#2,D0
		BLT	ItsHello
		BGT	ItsGoodbye
		BSR	FindSender		;who sent the message?
		Cmp.W	#-1,D0			;anybody we know?
		BNE	@0
		move.W	#'?',-(SP)		;we don't know who
		MOVE.L	OutputhTE(A5),-(SP)
		_TEKey
		BRA	@1
@0		AddQ	#5,A0			;print the name
		BSR	AddMyStr
@1		move.W	#':',-(SP)
		move.L	OutputhTE(A5),-(SP)
		_TEKey
		LEA	MyRequestBlock(A5),A0	;print the message
		BSR	AddMyStr
		move.W	#13,-(SP)		;and a CR
		move.L	OutputhTE(A5),-(SP)
		_TEKey				;and fall into MySendResponse
;
; We must send responses to requests, even though (in this case) we
; don't really have anything to return.  If we did want to return something,
; we can just put it in OutBDSBuffer.
;
MySendResponse	LEA	RequestParamBlock(A5),A0	;set up for a call to ATP
		move.W	#-11,ioRefNum(A0)	;use .MPP
		LEA	RespComplet,A1		;call who completion when we're done
		move.L	A1,ioCompletion(A0)
		move.W	#252,csCode(A0)		;we're doing a SendResponse
		move.B	MySocket(A5),+28(A0)	;say who's responding
		move.B	#1,+29(A0)		;try more than once
;
; +30, the address, still has the address to answer to.
; +46 still has the transaction ID to use.
;
		LEA	OutBDS(A5),A1		;setup the BDS's
		move.L	A1,+40(A0)
		move.B	#1,+44(A0)		;one responses expected
		move.B	#1,+45(A0)		;one element in BDS
		LEA	OutBDSBuffer(A5),A1	;setup the BDS, for responses
		move.L	A1,OutBDS+2(A5)
		move.W	#256,OutBDS(A5)
		_Control	,ASYNC
		BRA	ChoiceReturn
;
;  Someone has just walked in.  We must add him to our names list.
;  Note that, while we can get the nodeID and network directly from
;  the request, we have to get the socket as part of the userbytes,
;  since the socket making the request is not necessarily the same as the
;  socket that will receive our requests
;
ItsHello	move.W	MyNameCount(A5),D0	;add a new name
		AddQ	#1,D0
		move.W	D0,MyNameCount(A5)
		subq	#1,D0
		AndI.L	#$FFFF,D0		;point to where to put the name
		mulU	#22,D0
		AndI.L	#$FFFF,D0
		LEA	MyNames(A5),A1
		Add.L	D0,A1
		move.L	RequestParamBlock+30(A5),(A1)	;copy over the address
		move.B	RequestParamBlock+20(A5),+3(A1)	;replace the socket
		addQ	#5,A1			;copy over the name
		move.L	#16,D0
		LEA	MyRequestBlock(A5),A0
		_BlockMove
		LEA	MyStr1,A0		;add to the text
		BSR	Writeinout
		BRA	MySendResponse		;somebody walking in
;
;  We've gotten a goodbye message, someone quitting the program.
;   This removes a name from our list of legal chatters
;
ItsGoodbye	
		BSR	FindSender		;who left?
		Cmp.W	#0,D0			;is it anybody we know?
		BLT	@900
		move.W	MyNameCount(A5),D1	;decrement the count
		subq	#1,D1
		move.W	D1,MyNameCount(A5)
@100		Cmp.W	MyNameCount(A5),D0		;was it the last?
		BGE	@900
		move.W	D0,-(SP)		;move down one
		move.L	A0,A1
		Add.L	#22,A0
		move.L	#22,D0
		_BlockMove
		move.W	(SP)+,D0
		AddQ	#1,D0
		BRA	@100
@900		LEA	MyStr2,A0		;add to the text
		BSR	Writeinout
		BRA	MySendResponse
;
; This is called to announce an entry or an exit.  It assumes that
;  the RequestBlock is a string with the name of the person entering or exiting
;
WriteInOut	move.L	A0,-(SP)		;save the action
		LEA	MyRequestBlock(A5),A0	;print the message of walking in
		BSR	AddMyStr
		move.W	#':',-(SP)
		move.L	OutputhTE(A5),-(SP)
		_TEKey
		Move.L	(SP)+,A0
		BSR	AddMyStr
		move.W	#13,-(SP)		;and a CR
		move.L	OutputhTE(A5),-(SP)
		_TEKey
		move.L	MyWindow(A5),-(SP)	;redraw the names window
		_Setport
		PEA	NameRect
		_InvalRect
		RTS
;
; This takes a string, and puts it on the output screen's textedit record
;
AddMyStr	move.B	(A0),D0			;add the str pointed to by A0
		AndI.L	#$FF,D0
		Cmp.w	#0,D0
		BEQ.S	@0
		PEA	+1(A0)
		move.L	D0,-(SP)
		move.L	OutputhTE(A5),-(SP)
		_TEInsert
;
; Make sure that everythings okay
;
@0		Clr.L	D1			;Get the length of the text
		Move.L	OutputhTE(A5),A0	;Point to the text pointer
		Move.L	(A0),A0			;Point to the text
		Move.W	teNLines(A0),D1		;Get the number of lines
		Clr.L	D0			;Get number of lines in the screen
		Move.W	View2+4,D0
		Sub.W	View2,D0
		DivU	teLineHite(A0),D0
		Sub.W	D0,D1			;More than we've got space for?
		BLE.S	@900
;
; The Screen overflowed!  Delete the top lines!
;
		move.W	D1,-(SP)		;save the D1
		MulU	teLineHite(A0),D1	;see how much to move by
		move.W	D1,-(SP)		;and save this off, too!
		
		neg.W	D1
		Clr.W	-(SP)			;scroll things
		move.W	D1,-(SP)
		move.L	OutputhTE(A5),-(SP)
		_TEScroll
		Move.L	OutputhTE(A5),A0
		Move.L	(A0),A0
		Clr.L	teViewRect(A0)		;make the screen "invisible"
		Clr.L	teViewRect+4(A0)
		
		move.W	(SP)+,D1		;scroll back
		Clr.W	-(SP)
		move.W	d1,-(SP)
		move.L	OutputhTE(A5),-(SP)
		_TEScroll
		
		Move.L	OutputhTE(A5),A0
		Move.L	(A0),A0
		move.w	(SP)+,D1
		MulU	#2,D1			;get as many lines as needed
		Clr.L	D0			;Get the pointers to the start of the
		Move.W	teLines(A0),D0		;  lines to delete
		Move.L	D0,-(SP)
		Move.W	teLines(A0,D1.W),D0
		Move.L	D0,-(SP)
		Move.L	OutputhTE(A5),-(SP)
		_TESetSelect
		Move.L	OutputhTE(A5),-(SP)
		_TEDelete
		Move.L	#$FFFF,-(SP)		;Set selection to end of text
		Move.L	#$FFFF,-(SP)
		Move.L	OutputhTE(A5),-(SP)
		_TESetSelect
		Move.L	OutputhTE(A5),A0	;And go back to drawing the teRec
		Move.L	(A0),A0
		move.L	teDestRect(A0),teViewRect(A0)
		move.L	teDestRect+4(A0),teViewRect+4(A0)
@900		RTS
;
; This is called when a response has been sent. (event#15)
;  It then will queue up the next request to be received
;
EndResp		BSR	MyGetRequest		;the response is sent, so start next request
		BRA	ChoiceReturn
;
; FindSender looks through our list, and points A0 at the mini-tuple,
; Giving the count in D0  (D0=-1 if no finding)
;
FindSender	LEA	MyNames(A5),A0		;who's there?
		Clr.W	D0
		move.B	ReQuestParamBlock+32(A5),D1	;get their node
@0		cmp.W	MyNameCount(A5),D0
		BLT.S	@1
		move.w	#-1,D0			;sorry, no such luck
		RTS
@1		Cmp.b	+2(A0),D1			;is this the one?
		BNE	@2
		RTS
@2		Add.L	#22,A0			;nope, try next
		AddQ	#1,D0
		BRA	@0
;
; The next four routines are called when our asynchronous control calls
;  are finished.
;
; In order to minimize unexpected conflicts (since we have no way of
;  knowing what will be happening when the control calls finish),
;  our completion routines will only post our application-defined events.
;  We will then handle the network events as part of our regular main event loop.
;
RequestComplet	movem.L	D0/A0,-(SP)
		move.L	#12,A0				;post a "GotRequest" event
		_PostEvent
		movem.L	(SP)+,D0/A0
		RTS
WhoComplet	movem.L	D0/A0,-(SP)
		move.L	#13,A0				;post a "who finished" event
		_PostEvent
		movem.L	(SP)+,D0/A0
		RTS
MessComplet	movem.L	D0/A0,-(SP)
		move.L	#14,A0				;post a "message finished" event
		_PostEvent
		movem.L	(SP)+,D0/A0
		RTS
RespComplet	movem.L	D0/A0,-(SP)
		move.L	#15,A0				;post a "response finished" event
		_PostEvent
		movem.L	(SP)+,D0/A0
		RTS
;
; While we're getting a list of chatters, or broadcasting, we
; don't want another broadcast to start up.  This will wipe it out
;
DisableATalk	move.L	MyButton(A5),-(SP)	;disable the button
		move.W	#255,-(SP)
		_HiLiteControl
		Clr.W	MyActiveFlag(A5)
		RTS
;
; This allows broadcasts to start again
;
EnableATalk	move.L	MyButton(A5),-(SP)	;enable the button
		Clr.W	-(SP)
		_HiLiteControl
		move.W	#$FFFF,MyActiveFlag(A5)
		RTS
; ---------------------------- Data Starts Here -------------------------

EventRecord					; NextEvent's Record
 What: 			DC 	0		; Event number
 Message:		DC.L	0		; Additional information
 When:			DC.L	0		; Time event was posted 
 Point:			DC.L	0		; Mouse coordinates
 Modify:		DC 	0		; State of keys and button
 WWindow:		DC.L	0		; Find Window's Result

DStorage		DCB.W	DWindLen,0	; Storage For Dialog
DeskName		DCB.W	16,0		; Desk Accessory's Name
MyBounds		DC 	28,4,308,508	; Drag Window's Bounds
ViewRect		DC 	226,1,248,320	; Text Record's View Rect
DestRect		DC 	229,4,245,317	; Text Record's Dest Rect
View2			DC	5,4,215,305
Dest2			DC	5,4,215,305
NameRect		DC	5,310,215,405	; window for names
ItemHit			DC	0		; Item clicked in dialog
Tempbox			dc.L	0,0
TempItem		dc.L	0
Temptype		dc.w	0
;
SearchEntity		dc.b	1,'=',7,'Chatter',1,'*'	;get all chatters!
ChatType		dc.b	7,'Chatter',1,'*'
			.align	2
			string_format	2
MyStr1			dc.b	'Walking in'
			.align	2
MyStr2			dc.b	'Walking out'
			.align	2
			string_format	0
;
;------------------------- Nonrelocatable Storage -----------------------

; Variables declared using DS are placed in a global space relative to
; A5.  When these variables are referenced, A5 must be explicitly mentioned.

WindowStorage		DS.B	WindowSize	; Storage for Window
NBPParamBlock		DS.B	100		; storage for our control calls
ATPParamBlock		DS.B	100		; storage for the ATP calls
RequestParamBlock	DS.B	100		; storage for the GetRequest calls
MyEntity		DS.B	100		; storage for our name in the register
MyEntFlag		DS.W	1
MySearchBlock		DS.B	SBlockSize	; we look here for names
MyRequestBlock		DS.B	SBlockSize	; here messages come in
MyMessageBlock		DS.B	SBlockSize	; here messages go out
MyhTE			DS.L	1		; here is the pointer to our texteditor
OutputhTE		DS.L	1		; here's the pointer to the screen
MyWindow		DS.L	1		; pointer to our window
MyAppleMenu		DS.L	1		; handle to our apple menu
MyFileMenu		DS.L	1		; handle to our file menu
MyEditMenu		DS.L	1		; handle to our edit menu
Quitflag		DS.W	1		; a flag to quit!
MySocket		DS.W	1		; our socket is here
TempStr			DS.B	256		; a temporary string
MyNames			DS.B	300		; Keep the real table here
MyNameCount		DS.W	1		;how many names do we have
InBDS			DS.L	3		;we've got one incoming BDS
OutBDS			DS.L	3		;and one outgoing
InBDSBuffer		DS.B	256		;one page worth buffers
OutBDSBuffer		DS.B	256
Broadcastflag		DS.W	1		;set to -1 when broadcasting
NextBroadcast		DS.W	1		;next count to broadcast to
NeedHello		DS.W	1		;say hello to everybody flag
MyButton		DS.L	1		;button for transmitting
MyActiveFlag		DS.W	1		;can we transmit now?

			End
