// Copyright 1990 Waldemar Horwat.
// Permission is granted for noncommercial use of this code.

#include <Types.h>
#include <Memory.h>
#include <Errors.h>
#include <stdio.h>
#include "HString.h"

/**/
/* Split the path name in pathName into the volume name (without the colon) that is	*/
/* stored in volumeName and everything following it, which is stored in			*/
/* partialPathName.  A colon is prepended to partialPathName if it didn't already have	*/
/* one.  Both volumeName and partialPathName may be nil, in which case they are not	*/
/* returned.  Both volumeName and partialPathName can alias with pathName.		*/
/**/
OSErr splitVolumeName(const String &pathName, String *volumeName, String *partialPathName)
  {
  String::err=0;
  int length=pathName.length();

  for (int i=0; i<length && pathName[i]!=':'; i++);
  if (i==length)
    {
    if (volumeName) volumeName->clear();
    if (partialPathName)
      {
      *partialPathName=pathName;
      (*partialPathName)^=':';
      }
    }
  else
    {
    String pathName2=pathName;
    if (volumeName) *volumeName=pathName2.substr(0,i);
    if (partialPathName) *partialPathName=pathName2.substr(i,length-i);
    }
  return String::err;
  }


/**/
/* Split the path name in partialPathName into the last name (optionally followed by a	*/
/* colon, which is removed) that is stored in fileName and everything preceding it,	*/
/* which is stored in dirName.  Both fileName and dirName may be nil, in which case	*/
/* they are not returned.  Both fileName and dirName can alias with partialPathName.	*/
/**/
OSErr splitPartialPathName(const String &partialPathName, String *dirName, String *fileName)
  {
  String::err=0;
  OSErr err=0;
  String partialPathName2=partialPathName;
  int length=partialPathName2.length();

  if (length && partialPathName2[length-1]==':') length--;
  if (!length) err=bdNamErr;
  else
    {
    for (int i=length; i>0; --i)
      if (partialPathName2[i-1]==':') break;
    if (dirName) *dirName=partialPathName2.substr(0,i);
    if (fileName) *fileName=partialPathName2.substr(i,length-i);
    }
  if (!err) err=String::err;
  return err;
  }


OSErr parseFile(char *pathNameStr)
  {
  printf("\"%s\":\n",pathNameStr);

  String::err=0;
  String pathName=pathNameStr;
  OSErr err=String::err;
  String volumeName;
  String partialPathName;
  String fileName;

  if (!err)
    {
    err=splitVolumeName(pathName,&volumeName,&partialPathName);
    if (!err)
      {
      // I am not sure this is right; printf could allocate memory?
      printf("  VolumeName=\"%s\"\n",volumeName.cString());
      while (partialPathName.length())
        {
        err=splitPartialPathName(partialPathName,&partialPathName,&fileName);
        if (err)
	  {
	  if (err==bdNamErr) err=0;
	  break;
	  }
        printf("  DirName=\"%s\"; FileName=\"%s\"\n",partialPathName.cString(),fileName.cString());
        }
      }
    }
  return err;
  }


void main(int argc, char **argv)
  {
  int i;

  for (i=1; i<argc; i++)
    {
    OSErr err=parseFile(argv[i]);
    if (err) printf("(Error $%X)\n",err);
    printf("\n");
    }
  }
