 {	NOTE:  	This source code and the executable code created from it }
{			may be used freely for educational purposes.  Use in a }
{			commercial product requires a liscence from the University }
{			of Rochester. }
{  	------------------------------------------------------------------------- 	}
{	File:	DragGray.p }
{	Date:	6/25/90	}
{	By:		Jonathan Gary }
{	Purpose:	XFCN to do a "drag gray region" call. You can specify all of the same }
{			parameters as the toolbox call except for the action procedure. }
{ }
{	Usage:	get DragGray(theRgn,startPt,limit,slop,constraint)  }
{				where:	theRgn is a comma separated list of x-y points that}
{							for the region.}
{						startPt is the starting point of the drag.}
{						limit is the rectangle to limit the drag to.}
{						slop is the slop rect.}
{						constraint is either "hAxisOnly","vAxisOnly" or "noConstraint"}
{							and specifies constaint of motion.	 }
{			it will a point representing the h and v offsets made by the drag. }



UNIT UDragGray;

INTERFACE
USES
	HyperXCMD, JEGXcmdLib;

PROCEDURE Main (p: XCMDPtr);

IMPLEMENTATION

CONST
	applScratch = $A78;

TYPE
	TActionRec = RECORD
			xPtr: XCMDPtr;
			msg: Str255;
		END;
	TPActionRec = ^TActionRec;
	THActionRec = ^TPActionRec;

	TScratch = PACKED ARRAY[1..12] OF Byte;

{**********************************************************}
{	DoActionMsg executes actionMsg.  It must declared at the}
{	outermost level to be accessible by the "@" operator. }
PROCEDURE DoActionMsg;
	VAR
		theRefCon: LONGINT;
		actionRec: THActionRec;
	BEGIN
		actionRec := THActionRec(applScratch);
		IF actionRec^^.msg <> '' THEN
			SendCardMessage(actionRec^^.xPtr, actionRec^^.msg);
	END;

{**********************************************************}
PROCEDURE Main;
	VAR
		actionMsg: Str255;
		gPtr: XCMDPtr;



{**********************************************************}
{ Fail is a general bail out routine }
	PROCEDURE Fail (msg: Str255);
		BEGIN
			p^.returnValue := PasToZero(p, msg);
			EXIT(Main);
		END;

{**********************************************************}
{ CheckParams ensures the use has called BigAlert correctly }

	PROCEDURE CheckParams (p: XCMDPtr);

		BEGIN
			WITH p^ DO
				BEGIN
					IF (paramCount < 5) OR (paramCount > 6) THEN
						Fail('Error: five parameters needed')
				END;
		END;

{**********************************************************}
{	ParseConstaint turns the constants in  the axis parameter}
{	into the appropriate constants }
	FUNCTION ParseConstraint (strH: Handle): INTEGER;
		VAR
			hState: SignedByte;
			axis: INTEGER;
			temp: Str255;
		BEGIN
			hState := HGetState(strH);
			HLock(strH);
			ZeroToPas(p, strH^, temp);
			IF StringEqual(p, temp, 'hAxisOnly') THEN
				axis := hAxisOnly
			ELSE IF StringEqual(p, temp, 'vAxisOnly') THEN
				axis := vAxisOnly
			ELSE IF StringEqual(p, temp, 'noConstraint') THEN
				axis := noConstraint
			ELSE
				Fail('Error: Constraint parameter must be "hAxisOnly", "vAxisOnly" or "noConstraint"');
			ParseConstraint := axis;
		END;

{**********************************************************}
{ DragGray is the main procedure }

	PROCEDURE DragGray;
		VAR
			theRgn: RgnHandle;
			startPt: Point;
			limit, slop: Rect;
			dragResult: LONGINT;
			h, v: LONGINT;
			temp1, temp2: Str255;
			axis: INTEGER;
			actionRec: TActionRec;
			saveApplScr: TScratch;
			acPtr: THActionRec;
		BEGIN
			CheckParams(p);	{ check syntax }
			actionRec.xPtr := p;
			actionRec.msg := '';
			theRgn := BuildRgn(p, p^.params[1]);
			BuildPt(p, startPt, p^.params[2]);
			BuildRect(p, limit, p^.params[3]);
			BuildRect(p, slop, p^.params[4]);
			axis := ParseConstraint(p^.params[5]);
			IF p^.paramCount = 6 THEN
				ZeroToPas(p, p^.params[6]^, actionRec.msg);
			BlockMove(POINTER(applScratch), @saveApplScr, SizeOf(TScratch));
			acPtr := THActionRec(applScratch);
			acPtr^ := @actionRec;
			dragResult := DragGrayRgn(theRgn, startPt, limit, slop, axis, @DoActionMsg);
			BlockMove(@saveApplScr, POINTER(applScratch), SizeOf(TScratch));
			h := HiWord(dragResult);
			v := LoWord(dragResult);
			NumToString(h, temp1);
			NumToString(v, temp2);
			p^.returnValue := PasToZero(p, Concat(temp2, ',', temp1));
			DisposeRgn(theRgn);								{ clean up! }
		END; {BigAlert}

	BEGIN	{main}
		DragGray;
	END;	{main}


END. {UNIT}